/*   contrib/m_ojoin.c
 *   Copyright (C) 2002 Hybrid Development Team
 *   Copyright (C) 2004 ircd-ratbox Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_ojoin.c 27 2008-05-08 05:58:51Z Celestine.sensei $
 */

#include "stdinc.h"
#include "ratbox_lib.h"
#include "struct.h"
#include "channel.h"
#include "client.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "s_serv.h"
#include "s_newconf.h"
#include "send.h"
#include "whowas.h"
#include "match.h"
#include "hash.h"
#include "parse.h"
#include "modules.h"


static int mo_ojoin(struct Client *client_p, struct Client *source_p, int parc, const char *parv[]);


struct Message ojoin_msgtab = {
	"OJOIN", 0, 0, 0, MFLG_SLOW,
	{mg_unreg, mg_not_oper, mg_ignore, mg_ignore, mg_ignore, {mo_ojoin, 2}}
};

mapi_clist_av1 ojoin_clist[] = { &ojoin_msgtab, NULL };

DECLARE_MODULE_AV1(ojoin, NULL, NULL, ojoin_clist, NULL, NULL, "$Revision: 25127 $");

/*
** mo_ojoin
**      parv[0] = sender prefix
**      parv[1] = channel
*/
static int
mo_ojoin(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
{
	struct Channel *chptr;
	const char *prefix = ""; 
	char modeletter = '\0';
	const char *name = parv[1];
	unsigned int flags = CHFL_PEON;
	

	/* admins only */
	if(!IsOperAdmin(source_p))
	{
		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "ojoin");
		return 0;
	}

	/* XXX - we might not have CBURSTed this channel if we are a lazylink
	 * yet. */

	switch(*name)
	{
		case '@':
			prefix = "@";
			flags = CHFL_CHANOP;
			modeletter = 'o';
			++name;
			break;
#ifdef HALFOPS
		case '%':
			prefix = "%";
			flags = CHFL_HALFOP;
			modeletter = 'h';
			++name;
			break;
#endif
		case '+':
			prefix = "+";
			flags = CHFL_VOICE;
			modeletter = 'v';
			++name;
			break;
#ifdef CHANAQ
		case '~':
			prefix = "~";
			flags = CHFL_OWNER;
			modeletter = 'q';
			++name;
			break;
		case '!':
			prefix = "&";
			flags = CHFL_PROTECTED;
			modeletter = 'a';
			++name;
			break;
#endif
		case '&':
		case '#':
			prefix = "";
			flags = CHFL_PEON;
			modeletter = '\0';
			break;

		default:
			sendto_one_numeric(source_p, ERR_NOSUCHCHANNEL,
				   form_str(ERR_NOSUCHCHANNEL), name);
			return 0;		
	}

	if((chptr = find_channel(name)) == NULL)
	{
		sendto_one_numeric(source_p, ERR_NOSUCHCHANNEL,
				   form_str(ERR_NOSUCHCHANNEL), name);
		return 0;
	}
	else if(IsMember(source_p, chptr))
	{
		sendto_one(source_p, ":%s NOTICE %s :Please part %s before using OJOIN",
			   me.name, source_p->name, name);
		return 0;
	}
	else {
		add_user_to_channel(chptr, source_p, flags);
		if(chptr->chname[0] == '#')
		{
			sendto_server(client_p, chptr, NOCAPS, NOCAPS,
				      ":%s SJOIN %ld %s + :%s%s",
				      me.name, (long) chptr->channelts, chptr->chname, prefix, source_p->name);
		}
		sendto_channel_local(ALL_MEMBERS, chptr, ":%s!%s@%s JOIN %s",
				     source_p->name,
				     source_p->username, source_p->host, chptr->chname);
		if(modeletter != '\0')
			sendto_channel_local(ALL_MEMBERS, chptr, ":%s MODE %s +%c %s",
					     me.name, chptr->chname, modeletter, source_p->name);

		/* send the topic... */
		if(chptr->topic != NULL)
		{
			sendto_one(source_p, form_str(RPL_TOPIC), me.name,
				   source_p->name, chptr->chname, chptr->topic->topic);
			sendto_one(source_p, form_str(RPL_TOPICWHOTIME), me.name,
				   source_p->name, chptr->chname, chptr->topic->topic_info, chptr->topic->topic_time);
		}

		source_p->localClient->last_join_time = rb_current_time();
		channel_member_names(chptr, source_p, 1);
	}

	return 0;
}
